#include "LogitechCMReplicant.h"
#include <Message.h>
#include <Deskbar.h>
#include <Messenger.h>
#include <InterfaceDefs.h>

/**** constructor ****/
LogitechCMReplicant::LogitechCMReplicant()
:BView(BRect(0,0,7,15), "LogitechCMReplicant", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_NAVIGABLE)
{
}

/**** constructor with message ****/
LogitechCMReplicant::LogitechCMReplicant(BMessage *archive)
: BView(archive)
{
}

/**** destructor ****/
LogitechCMReplicant::~LogitechCMReplicant()
{
}

/**** archiving ****/
LogitechCMReplicant *LogitechCMReplicant::Instantiate(BMessage *data)
{
	if (!validate_instantiation(data,A_RC_CLASS_NAME))
		return NULL;
	return (new LogitechCMReplicant(data));
}

status_t LogitechCMReplicant::Archive(BMessage *data, bool deep = true) const
{
	BView::Archive(data, deep);
	
	data->AddString("add_on",A_RC_APP_SIGNATURE);
	data->AddString(A_RC_CLASS,A_RC_CLASS_NAME);

	return B_OK;
}

/**** attached to window ****/
void LogitechCMReplicant::AttachedToWindow(void)
{
	BMessage	message(DEFINE_REPLICANT_MSG);
	BMessenger	messenger(this);

	// default define
	_verrNum = false;
	_capsLock = false;
	_scrollLock = false;
	
   	// send messenger for access by application to replicant view
   	message.AddMessenger("replicant-messenger",messenger);   	
	(BMessenger(A_RC_APP_SIGNATURE)).SendMessage(&message);

	// define the VerrNum state
//	uint32 locks = modifiers();
//	locks = locks | B_SCROLL_LOCK;
//	set_keyboard_locks(locks);

	// define the key state
	message.AddInt32("modifiers",modifiers());
	ResolveKeyPressed(&message);
	
	BView::AttachedToWindow();
}

/**** Message received ****/
void LogitechCMReplicant::MessageReceived(BMessage *message)
{

	switch(message->what)
	{
		case B_KEY_DOWN:
		case B_UNMAPPED_KEY_DOWN:
			ResolveKeyPressed(message);
			break;
		default:
			BView::MessageReceived(message);
	}
}

/**** drawing ****/
void LogitechCMReplicant::Draw(BRect updateRect)
{
	BRect		bounds = Bounds();
	rgb_color	deskbarcolor = tint_color(ui_color(B_MENU_BACKGROUND_COLOR),B_DARKEN_1_TINT);
	
	SetHighColor(deskbarcolor);
	FillRect(bounds);
	
	// pour le vernumm
	SetHighColor(deskbarcolor);
	if(_verrNum)
		SetHighColor(0,255,0);

	// couleur pleine
	SetLowColor(HighColor());
	FillRect(BRect(2,1,5,4));

	// dessiner le cadre
	SetHighColor(tint_color(LowColor(),B_LIGHTEN_2_TINT));
	StrokeLine(BPoint(1,0),BPoint(6,0));	
	StrokeLine(BPoint(1,0),BPoint(1,5));	

	SetHighColor(tint_color(LowColor(),B_DARKEN_2_TINT));
	StrokeLine(BPoint(1,5),BPoint(6,5));	
	StrokeLine(BPoint(6,0),BPoint(6,5));	
	
	// pour le capslock
	SetHighColor(deskbarcolor);
	if(_capsLock)
		SetHighColor(255,0,0);

	// couleur pleine
	SetLowColor(HighColor());
	FillRect(BRect(2,7,5,9));

	// dessiner le cadre
	SetHighColor(tint_color(LowColor(),B_LIGHTEN_2_TINT));
	StrokeLine(BPoint(1,6),BPoint(6,6));	
	StrokeLine(BPoint(1,6),BPoint(1,10));	

	SetHighColor(tint_color(LowColor(),B_DARKEN_2_TINT));
	StrokeLine(BPoint(1,10),BPoint(6,10));	
	StrokeLine(BPoint(6,6),BPoint(6,10));
	
	// pour le scrolllock
	SetHighColor(deskbarcolor);
	if(_scrollLock)
		SetHighColor(0,0,255);

	// couleur pleine
	SetLowColor(HighColor());
	FillRect(BRect(2,12,5,14));

	// dessiner le cadre
	SetHighColor(tint_color(LowColor(),B_LIGHTEN_2_TINT));
	StrokeLine(BPoint(1,11),BPoint(6,11));	
	StrokeLine(BPoint(1,11),BPoint(1,15));	

	SetHighColor(tint_color(LowColor(),B_DARKEN_2_TINT));
	StrokeLine(BPoint(1,15),BPoint(6,15));	
	StrokeLine(BPoint(6,11),BPoint(6,15));
}

/**** mouse down : remove replicant ****/
void LogitechCMReplicant::MouseDown(BPoint point)
{
	uint32 		Button = 0;
	
	GetMouse(&point, &Button);
	
	switch(Button) 
	{
	case B_SECONDARY_MOUSE_BUTTON:
		{
		// quit application
		BMessenger(A_RC_APP_SIGNATURE).SendMessage(B_QUIT_REQUESTED);

    	BDeskbar desk;

   		LockLooper();
   		desk.RemoveItem(A_RC_CLASS_NAME);
   		UnlockLooper();
		}
		break;
	case B_PRIMARY_MOUSE_BUTTON:
		break;
	default:
		BView::MouseDown(point);
	}
}

/**** manage key pressed ****/
void LogitechCMReplicant::ResolveKeyPressed(BMessage *message)
{
	_verrNum = (modifiers() & B_NUM_LOCK);
	_capsLock = (modifiers() & B_CAPS_LOCK);
	_scrollLock = (modifiers() & B_SCROLL_LOCK);
	
	Invalidate();
}
